/***************************************************************************
 *   (C) 2007 by Tomas Mandys <tomas.mandys at 2p dot cz>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation;             *
 *                                                                         *
 ***************************************************************************/

/* Header file for iptables xt_RTPPROXY target */


#ifndef _XT_RTPPROXY_H_target
#define _XT_RTPPROXY_H_target

#define XT_RTPPROXY_MAX_GATE 2

enum xt_rtpproxy_session_state {
	xt_rtpproxy_NONE,	/* ready for allocation */
	xt_rtpproxy_INIT1,	/* half initialized session, we know caller IP|learning */
	xt_rtpproxy_INIT2,	/* full initialized session, we know caller&callee IP|learning, ready to forward */
	xt_rtpproxy_FORWARD1,	/* at least one packet forwarded caller->callee or callee->caller */
	xt_rtpproxy_FORWARD2,	/* packets forwarded in both directions */
	xt_rtpproxy_DESTROYED,	/* destroyed by sockopt command */
	xt_rtpproxy_EXPIRED	/* expired, no packet for long time (expires_timeout) */
};
#define xt_rtpproxy_MAX_STATES (xt_rtpproxy_EXPIRED+1)

struct xt_rtpproxy_addr {
	/* protocol: UDP only */
	u_int32_t ip;
	u_int16_t port;
};

struct xt_rtpproxy_stat {
	u_int32_t packets_in;
	u_int32_t packets_out;
	u_int32_t bytes_in;
	u_int32_t bytes_out;
};

struct xt_rtpproxy_session {
	u_int8_t/*enum xt_rtpproxy_session_state*/ state;
	u_int16_t sess_id;
	struct {
		struct {
		/* we need keep both RTP&RTCP ports because two consecutive port may be translated by NAT to nonconsecutive ones */
			u_int16_t port;  /* port allocated at gate (for sockopt) */
			struct xt_rtpproxy_addr source;
			u_int32_t learning;
			u_int32_t last_packet_stamp;
		} stream[2];
		u_int8_t always_learn;
		struct xt_rtpproxy_stat stat;

	} gate[XT_RTPPROXY_MAX_GATE];
	u_int32_t expires;
	u_int32_t flags;

	struct xt_rtpproxy_session *prev, *next; /* for sockopt */
};

struct xt_rtpproxy_sockopt_switchboard {
	struct {
		struct xt_rtpproxy_stat stat;
		/* seek registration by info and sockopt */
		struct xt_rtpproxy_addr addr;
	} gate[XT_RTPPROXY_MAX_GATE];	
	u_int16_t max_sess;	/* max number of handled sessions */
	u_int16_t expires_timeout; /* when no packet passes through during expiration interval then session is closed */
	u_int16_t resurrection_timeout; /* time that the session is not possible activate after death */
	u_int32_t flags;
	struct {
		u_int32_t state[xt_rtpproxy_MAX_STATES];
		struct {
			u_int32_t alloc;
			u_int32_t alloc_failed;
			u_int32_t updated;
			u_int32_t destroyed;
			u_int32_t expired;
		} counter;
	} stat;
};

struct xt_rtpproxy_switchboard {
	struct xt_rtpproxy_sockopt_switchboard so; /* we need it for both identification and sockopt */
	struct xt_rtpproxy_session *sessions;
	/* session alloc list maintanance */
	int usage;
	struct xt_rtpproxy_switchboard *prev, *next;
};

struct xt_rtpproxy_target_info {
	struct xt_rtpproxy_addr gate[XT_RTPPROXY_MAX_GATE];
	u_int16_t max_sess;	/* max number of handled sessions */
	/* end of part for userspace comparision, if changes adjust userspacesize in libipt_RTPPROXY.c */
	u_int16_t expires_timeout; /* when no packet passes through during expiration interval then session is closed */
	u_int16_t resurrection_timeout; /* time that the session is not possible activate after death */

	u_int32_t libipt_flags; /* aux flags for libipt */
#ifdef KERNEL_64_USERSPACE_32
	/* placeholder */
	u_int64_t switchboard;
#else
	/* Used internally by the kernel */
	#ifdef __KERNEL__
	struct xt_rtpproxy_switchboard *switchboard;
	#else
	void *switchboard;
	#endif
#endif
};

#ifdef KERNEL_64_USERSPACE_32
	/* placeholder */
	typedef u_int64_t xt_rtpproxy_handle;
#else
	/* Used internally by the kernel */
	#ifdef __KERNEL__
	typedef struct xt_rtpproxy_switchboard* xt_rtpproxy_handle;
	#else
	typedef void* xt_rtpproxy_handle;
	#endif
#endif


#define XT_RTPPROXY_SO_BASE_CTL (64+1024+128)	/* unique socket command range */

#define XT_RTPPROXY_SO_SET_UPDATE_SESSION	(XT_RTPPROXY_SO_BASE_CTL + 0)
#define XT_RTPPROXY_SO_SET_DELETE_SESSION	(XT_RTPPROXY_SO_BASE_CTL + 1)
#define XT_RTPPROXY_SO_SET_MAX         	XT_RTPPROXY_SO_SET_DELETE_SESSION

#define XT_RTPPROXY_SO_GET_CONFIG		(XT_RTPPROXY_SO_BASE_CTL + 0)
#define XT_RTPPROXY_SO_GET_ALLOC_SESSION	(XT_RTPPROXY_SO_BASE_CTL + 1)
#define XT_RTPPROXY_SO_GET_MAX			XT_RTPPROXY_SO_GET_ALLOC_SESSION


/* userspace communication */
#define XT_RTPPROXY_SOCKOPT_VERSION 0x01
/* in flags */
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_SWITCHBOARD		0x0001
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_SESS_ID 			0x0002
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_RESET_GLOBAL_STAT		0x0004
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_FORCE_SWITCHBOARD_AUDIT	0x0008
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_RESET_SWITCHBOARD_STAT	0x0010
#define XT_RTPPROXY_SOCKOPT_FLAG_IN_RESET_SWITCHBOARD_PACKET_STAT	0x0020
/* out flags */
#define XT_RTPPROXY_SOCKOPT_FLAG_OUT_COUNT			0x0100
#define XT_RTPPROXY_SOCKOPT_FLAG_OUT_SWITCHBOARD		0x0200
#define XT_RTPPROXY_SOCKOPT_FLAG_OUT_SESSION			0x0400
#define XT_RTPPROXY_MAX_CMD (XT_RTPPROXY_SO_SET_MAX-XT_RTPPROXY_SO_BASE_CTL+1+XT_RTPPROXY_SO_GET_MAX-XT_RTPPROXY_SO_BASE_CTL+1)

/* error contants for xt_rtpproxy_handle.err_no */
#define XT_RTPPROXY_ERR_CANNOT_OPEN_SOCKET	(-100)
#define XT_RTPPROXY_ERR_BAD_VERSION		(-101)
#define XT_RTPPROXY_ERR_UNEXPECTED		(-102)
#define XT_RTPPROXY_ERR_OUT_OF_MEMORY		(-103)
#define XT_RTPPROXY_ERR_PARAM			(-104)

struct xt_rtpproxy_sockopt_header {
	u_int32_t version;
	u_int32_t flags;
};

struct xt_rtpproxy_sockopt_in_switchboard {
	struct xt_rtpproxy_addr gate[XT_RTPPROXY_MAX_GATE];
};

struct xt_rtpproxy_sockopt_in_sess_id {
	u_int16_t sess_id_min;
	u_int16_t sess_id_max;  /* borders included */
};

struct xt_rtpproxy_sockopt_num_session {
	u_int32_t num_switchboard;
	u_int32_t num_session;
};

struct xt_rtpproxy_sockopt_out_config {
	struct xt_rtpproxy_sockopt_header h;
	u_int32_t started;
	struct {
		u_int32_t ok;
		u_int32_t fail;
		u_int32_t perm;
		u_int32_t unknown;
		u_int32_t cmd[XT_RTPPROXY_MAX_CMD];
	} stat;
	struct xt_rtpproxy_sockopt_num_session total;
/*	if (flags & XT_RTPPROXY_SOCKOPT_FLAG_OUT_COUNT) {
		struct xt_rtpproxy_sockopt_num_session count;  // userspace sockopt need preallocate memory
	}
	if (flags & XT_RTPPROXY_SOCKOPT_FLAG_OUT_SWITCHBOARD) {
		do {
			struct xt_rtpproxy_sockopt_switchboard data[];
			if data[].flags & XT_RTPPROXY_SOCKOPT_FLAG_OUT_SESSION {
				do {
					struct xt_rtpproxy_session sess[];
				} while sess[].flags & XT_RTPPROXY_SOCKOPT_FLAG_OUT_SESSION;
			}
		} while data[].flags & XT_RTPPROXY_SOCKOPT_FLAG_OUT_SWITCHBOARD;
	}
*/
};

#define XT_RTPPROXY_SOCKOPT_FLAG_SESSION_ADDR 0x01
#define XT_RTPPROXY_SOCKOPT_FLAG_SESSION_LEARNING_TIMEOUT 0x02

struct xt_rtpproxy_sockopt_in_alloc_session {
	struct {
		struct {
			u_int8_t flags;
			struct xt_rtpproxy_addr addr;
			u_int32_t learning_timeout;
		} stream[2];
		u_int8_t always_learn;
	} source[XT_RTPPROXY_MAX_GATE];
};

/* libipt function prototypes */
struct xt_rtpproxy_handle {
	int sockfd;
	int err_no; /* see XT_RTPPROXY_ERR_xxx */
	char *err_str;
	struct xt_rtpproxy_sockopt_out_config info;
};


#ifdef __cplusplus
extern "C" {
#endif

int x_RTPPROXY_open(struct xt_rtpproxy_handle *handle);
void xt_RTPPROXY_close(struct xt_rtpproxy_handle *handle);

int xt_RTPPROXY_get_switchboards(
	struct xt_rtpproxy_handle *handle,
	struct xt_rtpproxy_sockopt_in_switchboard *in_switchboard_id,	/* NULL for all switchboards */
	struct xt_rtpproxy_sockopt_in_sess_id *in_sess_id,		/* NULL for all sessions */
	unsigned int flags,						/* required action */
	struct xt_rtpproxy_switchboard **out_switchboard		/* NULL if not switchboard info demanded */
);

void xt_RTPPROXY_release_switchboards(struct xt_rtpproxy_handle *handle, struct xt_rtpproxy_switchboard *switchboard);

#define xt_RTPPROXY_get_info(handle, flags) xt_RTPPROXY_get_switchboards(handle, NULL, NULL, flags, NULL)

int xt_RTPPROXY_alloc_session(
	struct xt_rtpproxy_handle *handle,
	struct xt_rtpproxy_sockopt_in_switchboard *in_switchboard_id,	/* NULL to alloc at any switchboard */
	struct xt_rtpproxy_sockopt_in_alloc_session *in_session,	/* identify initial state, at least one learning timeout or source ip must be specified! */
	struct xt_rtpproxy_sockopt_switchboard *out_switchboard_id,	/* usefull when in_switchboard_id is NULL, otherwise can be NULL */
	struct xt_rtpproxy_session *out_session
);

int xt_RTPPROXY_update_session(
	struct xt_rtpproxy_handle *handle,
	struct xt_rtpproxy_sockopt_in_switchboard *in_switchboard_id,	/* mandatory to identify updated switchboard */
	struct xt_rtpproxy_sockopt_in_sess_id *in_sess_id,		/* NULL to update all sessions of given switchboard */
	struct xt_rtpproxy_sockopt_in_alloc_session *in_session
);

int xt_RTPPROXY_delete_session(
	struct xt_rtpproxy_handle *handle,
	struct xt_rtpproxy_sockopt_in_switchboard *in_switchboard_id,	/* NULL to delete all switchboards */
	struct xt_rtpproxy_sockopt_in_sess_id *in_sess_id		/* NULL to delete all sessions of given switchboard */
);

#ifdef __cplusplus
}
#endif


#endif /*_XT_RTPPROXY_H_target*/
